@@ -12,11 +12,13 @@ import math
 import numbers
 import operator
 import os
+from time import sleep
 import platform
 import re
 import shlex
 import string
 import struct
+import webbrowser
 import subprocess
 import sys
 import time
@@ -39,7 +41,7 @@ __all__ = ['get_session', 'set_session', 'get_session_value', 'is_autocast_enabl
 
            'GetImageMode', 'split_path', 'make_dir_if_need', 'sanitize_path', 'ShortcutMode', 'adaptive_format', 'num_cpus',
            'get_args_spec', 'get_gpu_memory_map', 'get_memory_profile', 'get_gpu_memory_map', 'dtype', 'red_color', 'green_color', 'cyan_color', 'blue_color', 'orange_color',
-           'gray_color', 'yellow_color']
+           'gray_color', 'yellow_color','open_browser','launchTensorBoard','launchMLFlow']
 
 # In some cases, these basic types are shadowed by corresponding
 # top-level values.  The underscore variants let us refer to these
@@ -300,9 +302,9 @@ def adaptive_format(num: numbers.Number, prev_value: Union[numbers.Number, Itera
             return '{0:.3%}'.format(num)
         elif len(prev_value) > 0:
             digit = int(np.array([builtins.abs(builtins.min(math.log10(builtins.abs(s)), 0)) + 3 if s != 0 else 0 for s in prev_value]).mean())
-            if digit > 5:
+            if digit > 6:
                 return '{0:{1}}'.format(num, '.3e')
-            elif digit < 3:
+            elif digit < 4:
                 return '{0:{1}}'.format(num, '.3f')
             elif digit <= 5:
                 return '{0:{1}}'.format(num, '.4f')
@@ -320,7 +322,7 @@ def adaptive_format(num: numbers.Number, prev_value: Union[numbers.Number, Itera
                 return '{0:,}'.format(num)
             elif 1.2 >= num >= 0.001 or -0.001 >= num >= -1.2 or num == 0:
                 return '{0:.3%}'.format(num)
-            elif num > 1.2 or num < -1.2:
+            elif num > 4.2 or num < -4.2:
                 return '{0:{1}}'.format(num, '.3f')
             else:
                 return '{0:{1}}'.format(num, '.3e')
@@ -1814,3 +1816,21 @@ def is_instance(instance, check_class):
         if not inspect.isclass(check_class):
             print(red_color('Input check_class should a class, but {0}'))
         return False
+
+
+def open_browser(url,delay=0):
+    sleep(delay)
+    webbrowser.open_new(url)
+
+def launchTensorBoard():
+    make_dir_if_need(os.path.join(_SESSION.working_directory, 'Logs'))
+    print('tensorboard --logdir {0} --port {1}'.format(os.path.join(_SESSION.working_directory, 'Logs'),_SESSION.tensorboard_port))
+    os.system('tensorboard --logdir {0} --port {1}'.format(os.path.join(_SESSION.working_directory, 'Logs'),_SESSION.tensorboard_port))
+    return
+
+
+def launchMLFlow():
+    make_dir_if_need(os.path.join(_SESSION.working_directory, 'Logs'))
+    print('mlflow ui')
+    os.system('mlflow ui')
+    return
\ No newline at end of file
