@@ -16,6 +16,7 @@ import uuid
 from typing import Callable
 
 import numpy as np
+from trident.callbacks.callback_base import LambdaCallback, UnfreezeModelCallback
 
 from trident.backend.common import to_list, format_time, get_terminal_size, get_session, get_backend, \
     PrintException, OrderedDict, split_path, sanitize_path, adaptive_format, dtype, cyan_color, get_class
@@ -1031,14 +1032,14 @@ class ModelBase(object):
         raise NotImplementedError
 
     def trigger_when(self, when='on_batch_end', frequency=None, unit='batch', action=None):
-        lambdacallback=get_class('trident.callbacks.LambdaCallback')
-        new_callbacks =lambdacallback(when, frequency=frequency, unit=unit, action=action)
+
+        new_callbacks =LambdaCallback(when, frequency=frequency, unit=unit, action=action)
         self.with_callbacks(new_callbacks)
         return self
 
     def unfreeze_model_scheduling(self, frequency: int, unit='epoch', slice_from=None, slice_to=None, module_name=None):
-        unfreezeModelCallback = get_class('trident.callbacks.UnfreezeModelCallback')
-        self.callbacks.append(unfreezeModelCallback(frequency, unit, slice_from, slice_to, module_name=module_name))
+
+        self.callbacks.append(UnfreezeModelCallback(frequency, unit, slice_from, slice_to, module_name=module_name))
         return self
 
     def cpu(self):
