@@ -29,7 +29,8 @@ elif get_backend() == 'tensorflow':
     from trident.backend.tensorflow_backend import get_device
     from trident.backend.tensorflow_ops import to_numpy, to_tensor, arange, shuffle, cast, clip, sqrt, int_shape, concate, zeros_like, ones_like
 
-working_directory = get_session().working_directory
+ctx=get_session()
+working_directory = ctx.working_directory
 __all__ = ['RegularizationCallbacksBase', 'MixupCallback', 'CutMixCallback']
 
 
@@ -113,10 +114,16 @@ class MixupCallback(RegularizationCallbacksBase):
                     item = Unnormalize([0.485, 0.456, 0.406], [0.229, 0.224, 0.225])(to_numpy(item))
                     item = Unnormalize(0, 255)(item)
                     array2image(item).save(os.path.join(self.save_path, 'mixup_{0}.jpg'.format(get_time_suffix())))
+                    if ctx.enable_mlflow:
+                        ctx.mlflow_logger.add_image(os.path.join(self.save_path, 'mixup_{0}.jpg'.format(get_time_suffix())))
+
                 elif self.save_path is not None:
                     item = Unnormalize([0.485, 0.456, 0.406], [0.229, 0.224, 0.225])(to_numpy(item))
                     item = Unnormalize(0, 255)(item)
                     array2image(item).save(os.path.join(self.save_path, 'mixup_{0}.jpg'.format(get_time_suffix())))
+                    if ctx.enable_mlflow:
+                        ctx.mlflow_logger.add_image(os.path.join(self.save_path, 'mixup_{0}.jpg'.format(get_time_suffix())))
+
         mixed_x = None
         x = None
         y = None
@@ -233,12 +240,17 @@ class CutMixCallback(RegularizationCallbacksBase):
                     item = Unnormalize([0.485, 0.456, 0.406], [0.229, 0.224, 0.225])(to_numpy(item))
                     item = Unnormalize(0, 255)(item)
                     array2image(item).save(os.path.join(self.save_path, 'cutmix_{0}.jpg'.format(get_time_suffix())))
+                    if ctx.enable_mlflow:
+                        ctx.mlflow_logger.add_image(os.path.join(self.save_path, 'cutmix_{0}.jpg'.format(get_time_suffix())))
 
             elif self.save_path is not None:
                 for item in x:
                     item = Unnormalize([0.485, 0.456, 0.406], [0.229, 0.224, 0.225])(to_numpy(item))
                     item = Unnormalize(0, 255)(item)
                     array2image(item).save(os.path.join(self.save_path, 'cutmix_{0}.jpg'.format(get_time_suffix())))
+                    if ctx.enable_mlflow:
+                        ctx.mlflow_logger.add_image(os.path.join(self.save_path, 'cutmix_{0}.jpg'.format(get_time_suffix())))
+
         x = None
         y = None
 
