@@ -172,6 +172,8 @@ class _Context:
         self.enable_tensorboard=False
         self.summary_writer=None
         self.mlflow_logger = None
+        self.enable_mlflow = False
+
 
         self.locale = locale.getdefaultlocale()[0].lower()
 
@@ -185,6 +187,11 @@ class _Context:
         self.amp_available = False
         self.is_autocast_enabled = False
 
+        self.tensorboard_server='localhost'
+        self.tensorboard_port =6006
+        self.mlflow_server = 'localhost'
+        self.mlflow_port = 5000
+
         if 'COLAB_TPU_ADDR' in os.environ:
             self.tpu_address = 'grpc://' + os.environ['COLAB_TPU_ADDR']
             print('TPU is available.')
@@ -282,6 +289,10 @@ class _Context:
         self.enable_tensorboard=True
         self.summary_writer = summary_writer
 
+    def try_enable_mlflow(self,mlflow_logger):
+        self.enable_mlflow=True
+        self.mlflow_logger = mlflow_logger
+
     def regist_data_provider(self,data_provider ):
         if not hasattr(self._thread_local_info,'data_providers'):
             self._thread_local_info.data_providers=OrderedDict()
