@@ -1108,6 +1108,7 @@ class Iterator(object):
             self.sample_filter = sample_filter
             self.batch_sampler.sample_filter = self.sample_filter
 
+
     @property
     def data(self):
         return self._data
@@ -1277,14 +1278,11 @@ class Iterator(object):
             label = self.label.__getitem__(index % len(self.label)) if self.label is not None and len(self.label) > 0 else None
 
             unpair = self.unpair.__getitem__(index % len(self.unpair)) if self.unpair is not None and len(self.unpair) > 0 else None
-
-
-
             results = iteration_tools.flatten((data, label, unpair), iterable_types=(tuple))
             results = tuple([item for item in results if item is not None])
+            if len(returnData) > 0 and len(returnData) != len(self.get_datasets()):
+                raise ValueError("Flattened data should have same length as datasets")
 
-            if  len(returnData) >0 and  len(returnData) != len(results):
-                raise ValueError("Flattened data sh")
 
             for n  in range(len(self.get_datasets())):
                 ds=self.get_datasets()[n]
@@ -1305,6 +1303,7 @@ class Iterator(object):
             for i in range(len(returnData)):
                 threads[i].join()
 
+
             if self.signature is None or len(self.signature) == 0:
                 self.signature = Signature(name='data_provider')
                 for spec in returnData.key_list:
