@@ -1722,7 +1722,7 @@ class ImageMosaic(VisionTransform):
         gain (float): The constant multiplier.
     """
 
-    def __init__(self, output_size, keep_prob=0.7, name='image_mosaic', **kwargs):
+    def __init__(self, output_size=None, keep_prob=0.7, name='image_mosaic', **kwargs):
         super().__init__(name)
         self.output_size = output_size
         self.keep_prob = keep_prob
