@@ -849,7 +849,7 @@ class ShortCut2d(Layer):
         self.to(self.device)
 
     def build(self, input_shape: TensorShape):
-        if self._built == False:
+        if not self._built:
             if self.branch_from is not None:
                 for k, v in self.nodes.item_list:
                     if v.name == self.branch_from:
@@ -885,7 +885,6 @@ class ShortCut2d(Layer):
 
         branch1 = None
         if hasattr(self, 'branch_from_uuid') and self.branch_from_uuid is not None and self.branch_from_uuid in self.nodes.key_list:
-            tt = self.nodes.get(self.branch_from_uuid)
             branch1 = identity(self.nodes.get(self.branch_from_uuid)._output_tensor)
             if self.mode == 'add':
                 current = current + branch1
@@ -894,7 +893,7 @@ class ShortCut2d(Layer):
             elif self.mode == 'concate':
                 concate_list.append(branch1)
                 if len(concate_list) > 1:
-                    print('')
+                    pass
         try:
             if self.mode == 'concate':
                 x = concate(concate_list, axis=self.axis)
