@@ -17,7 +17,7 @@ import torch.nn as nn
 import torch.nn.functional as F  # import torch functions
 import torch.utils.hooks as hooks
 from torch._jit_internal import List
-from torch._six import container_abcs
+from collections import abc
 from torch.nn import Module
 from torch.nn import init
 from torch.nn.parameter import Parameter
