@@ -42,6 +42,9 @@ class HistoryBase(OrderedDict):
     def enable_tensorboard(self):
         return ctx.enable_tensorboard
 
+    @property
+    def enable_mlflow(self):
+        return ctx.enable_mlflow
 
     def regist(self,data_name:str):
         if data_name not in self:
@@ -64,6 +67,8 @@ class HistoryBase(OrderedDict):
                     ctx.summary_writer.add_scalar( self.name+"/"+data_name, value, global_step=step, walltime=time.time())
                 else:
                     ctx.summary_writer.add_scalar(self.training_name+ "/"+self.name + "/" + data_name, value, global_step=step, walltime=time.time())
+            if ctx.enable_mlflow:
+                    ctx.mlflow_logger.add_scalar( data_name, value, global_step=step, walltime=time.time())
 
     def reset(self):
         for i in range(len(self)):
