@@ -116,12 +116,12 @@ def plot_bbox(box, img, color=None, label=None, line_thickness=None, **kwargs):
             fontcolor = (0, 0, 0)
         if label and get_plateform() == 'windows':
             font = ImageFont.truetype(fonts[fontnames.index('Microsoft Sans Serif')], int(math.sqrt(img_shape[0] / 1000) * 10 + 1))
-        tf = max(tl - 1, 1)  # font thickness
-        t_size = draw.textsize(label, font=font)
-        c2 = c1[0] + t_size[0], c1[1] - t_size[1] - 3
-        offset = font.getoffset(label)
-        draw.rectangle((c1, c2), fill=color,width=2)
-        draw.text((c1[0], c1[1] - 2- offset[1] ), u'{0}'.format(label), fill=fontcolor, font=font)
+            tf = max(tl - 1, 1)  # font thickness
+            t_size = draw.textsize(label, font=font)
+            c2 = c1[0] + t_size[0], c1[1] - t_size[1] - 3
+            offset = font.getoffset(label)
+            draw.rectangle((c1, c2), fill=color,width=2)
+            draw.text((c1[0], c1[1] - 2- offset[1] ), u'{0}'.format(label), fill=fontcolor, font=font)
     except Exception as e:
         print('image_size', img_shape,box)
         print(e)
@@ -176,6 +176,8 @@ def tile_rgb_images(*imgs, row=3, save_path=None, imshow=False, legend=None, **k
         if save_path is not None:
             filename = save_path.format(suffix)
             plt.savefig(filename, bbox_inches='tight')
+            if ctx.enable_mlflow:
+                ctx.mlflow_logger.add_image(filename)
         if imshow:
             # plSize = fig.get_size_inches()
             # fig.set_size_inches((int(round(plSize[0] * 0.75, 0)), int(round(plSize[1] * 0.75, 0))))
@@ -364,6 +366,8 @@ def loss_metric_curve(losses, metrics,metrics_names,legend=None, calculate_base=
 
     if save_path is not None:
         plt.savefig(save_path, bbox_inches='tight')
+        if ctx.enable_mlflow:
+            ctx.mlflow_logger.add_image(save_path)
     plt.tight_layout()
     if imshow:
         if is_in_ipython():
