@@ -5,12 +5,20 @@ import math
 import random
 import time
 import uuid
+import os
+import sys
+import threading
+import subprocess
+import webbrowser
+from time import sleep
 from copy import deepcopy
 from itertools import count
-from typing import List, Tuple, Optional, Union, Callable, Any, Iterable, Mapping, TypeVar,Dict
+from typing import List, Tuple, Optional, Union, Callable, Any, Iterable, Mapping, TypeVar, Dict
 import numpy as np
 import torch
 from torch.distributions import Bernoulli, Categorical
+
+from trident import context
 from trident.backend.common import *
 from trident.backend.pytorch_backend import *
 from trident.backend.pytorch_ops import *
@@ -20,11 +28,15 @@ from trident.layers.pytorch_initializers import kaiming_normal, orthogonal
 from trident.misc.visualization_utils import loss_metric_curve
 from trident.optims.pytorch_trainer import Model, MuiltiNetwork
 from trident.reinforcement.utils import ReplayBuffer, Rollout, ActionStrategy
+from trident.loggers.history import HistoryBase
 
 import_or_install('gym')
 import gym
 
-_session = get_session()
+ctx = context._context()
+_backend = ctx.get_backend()
+working_directory = ctx.working_directory
+
 __all__ = ['PolicyBase', 'ActorCriticPolicy', 'DqnPolicy', 'PGPolicy', 'A2CPolicy', 'PPOPolicy']
 
 
@@ -225,7 +237,7 @@ class PolicyBase(Model):
             self.save_or_sync_weights()
 
     def play(self, num_episodes, batch_size=1, min_replay_samples=1, print_progess_frequency=5, training=True, train_timing='on_episode_start', train_every_nstep=1, repeat_train=1,
-             need_render=True):
+             need_render=True, **kwargs):
         if train_timing not in ['on_episode_start', 'on_step_end', 'on_step_start']:
             raise ValueError('Only on_episode_start,on_step_end are valid  train_timing options')
 
@@ -319,7 +331,23 @@ class PolicyBase(Model):
         self.env.close()
 
     def learn(self, num_episodes, batch_size=1, min_replay_samples=1, print_progess_frequency=5, train_timing='on_episode_start', train_every_nstep=1, repeat_train=1,
-              need_render=True):
+              need_render=True, **kwargs):
+        if ctx.enable_tensorboard:
+            for k, v in self.training_context.items():
+                if isinstance(v, HistoryBase):
+                    v.training_name = self.name
+
+            self.training_context['training_name'] = self.name
+            self.training_context['summary_writer'] = ctx.summary_writer
+
+            t1 = threading.Thread(target=launchTensorBoard, args=([]))
+            t1.setDaemon(True)
+            t1.start()
+            open_browser('http://{0}:{1}/'.format(ctx.tensorboard_server, ctx.tensorboard_port), 5)
+        if ctx.enable_mlflow:
+            ctx.mlflow_logger.start_run(run_id=self.execution_id)
+            open_browser('http://{0}:{1}/'.format(ctx.mlflow_server, ctx.mlflow_port), 5)
+
         self.training_context['repeat_train'] = repeat_train
         self.training_context['train_timing'] = train_timing
         self.training_context['train_every_nstep'] = train_every_nstep
@@ -386,12 +414,40 @@ class PolicyBase(Model):
     def do_on_batch_end(self):
         self.training_context['time_batch_progress'] += (time.time() - self.training_context['time_batch_start'])
         self.training_context['time_epoch_progress'] += (time.time() - self.training_context['time_batch_start'])
-        if (self.training_context['steps'] + 1) % _session.epoch_equivalent == 0:
-            if self.warmup > 0 and self.warmup == (self.training_context['steps'] + 1) // _session.epoch_equivalent:
+        if (self.training_context['steps'] + 1) % ctx.epoch_equivalent == 0:
+            if self.warmup > 0 and self.warmup == (self.training_context['steps'] + 1) // ctx.epoch_equivalent:
                 self.adjust_learning_rate(self.training_context['base_lr'])
                 self.warmup = 0
 
+    def with_tensorboard(self):
+        make_dir_if_need(os.path.join(working_directory, 'Logs'))
+        # check weather have tensorboard
+        if get_backend() == 'pytorch':
+            try:
+                from trident.loggers.pytorch_tensorboard import SummaryWriter
+                ctx.try_enable_tensorboard(SummaryWriter(os.path.join(working_directory, 'Logs')))
 
+            except Exception as e:
+                print('Tensorboard initialize failed, please check the installation status about Tensorboard.')
+                print(e)
+                PrintException()
+        elif get_backend() == 'tensorflow':
+            try:
+                from trident.loggers.tensorflow_tensorboard import SummaryWriter
+                ctx.try_enable_tensorboard(SummaryWriter(os.path.join(working_directory, 'Logs')))
+
+            except Exception as e:
+                print('Tensorboard initialize failed, please check the installation status about Tensorboard.')
+                print(e)
+                PrintException()
+        return self
+
+
+    def with_mlflow(self):
+        from trident.loggers.mlflow_logger import MLFlowLogger
+        ctx.try_enable_mlflow(MLFlowLogger())
+
+        return self
 def Critic(cls):
     """Actor Wrapper class that provides proxy access to an instance of actor model."""
 
@@ -465,13 +521,13 @@ class ActorCriticPolicy(MuiltiNetwork):
         self.replay_unit = replay_unit
         self.name = name
         self.action_strategy = action_strategy
-        self.state_shape=state_shape
+        self.state_shape = state_shape
         if self.state_shape is not None:
             self.add_network('actor', input_shape=self.state_shape, output=actor)
             self.add_network('critic', input_shape=self.state_shape, output=critic)
         else:
-            self.add_network('actor',inputs=self.get_observation(), output=actor)
-            self.add_network('critic',inputs=self.get_observation(), output=critic)
+            self.add_network('actor', inputs=self.get_observation(), output=actor)
+            self.add_network('critic', inputs=self.get_observation(), output=critic)
         self.setting_network()
         self.train_steps = -1
         self.frame_steps = -1
@@ -503,7 +559,10 @@ class ActorCriticPolicy(MuiltiNetwork):
         return self.env.action_space.samples()
 
     def get_rewards(self, action):
-        return self.env.step(action)
+        if hasattr(self,'done') and self.done:
+            self.env.reset()
+        else:
+            return self.env.step(action)
 
     def experience_replay(self, batch_size):
         return NotImplemented
@@ -601,7 +660,7 @@ class ActorCriticPolicy(MuiltiNetwork):
         else:
             raise ValueError('target_model and trained_model should be trident Model or Layer.')
 
-    def calculate_gae(self, values, next_values, rewards, dones, gamma: float, gae_lambda: float,**kwargs):
+    def calculate_gae(self, values, next_values, rewards, dones, gamma: float, gae_lambda: float, **kwargs):
         try:
             R = [0] * len(rewards)
             delta = to_tensor(rewards) + next_values * gamma - values
@@ -615,7 +674,7 @@ class ActorCriticPolicy(MuiltiNetwork):
         except Exception as e:
             print(e)
 
-    def update_actor(self, current_episode=0, current_step=0, num_episodes=100, repeat_train=1, train_timing=None, done=False, batch_size=1, accumulate_grads=False,**kwargs):
+    def update_actor(self, current_episode=0, current_step=0, num_episodes=100, repeat_train=1, train_timing=None, done=False, batch_size=1, accumulate_grads=False, **kwargs):
         is_collect_data = False
         for i in range(repeat_train):
             self.actor.train_model(None, None,
@@ -629,7 +688,7 @@ class ActorCriticPolicy(MuiltiNetwork):
                                    log_gradients=False, log_weights=False,
                                    accumulate_grads=accumulate_grads)
 
-    def update_critic(self, current_episode=0, current_step=0, num_episodes=100, repeat_train=1, train_timing=None, done=False, batch_size=1, accumulate_grads=False,**kwargs):
+    def update_critic(self, current_episode=0, current_step=0, num_episodes=100, repeat_train=1, train_timing=None, done=False, batch_size=1, accumulate_grads=False, **kwargs):
         is_collect_data = False
         for i in range(repeat_train):
             self.critic.train_model(None, None,
@@ -643,7 +702,7 @@ class ActorCriticPolicy(MuiltiNetwork):
                                     log_gradients=False, log_weights=False,
                                     accumulate_grads=accumulate_grads)
 
-    def training_model(self, current_episode=0, current_step=0, num_episodes=100, train_timing=None, done=False, batch_size=1, repeat_train=1,**kwargs):
+    def training_model(self, current_episode=0, current_step=0, num_episodes=100, train_timing=None, done=False, batch_size=1, repeat_train=1, **kwargs):
 
         is_collect_data = False
         for i in range(repeat_train):
@@ -668,120 +727,154 @@ class ActorCriticPolicy(MuiltiNetwork):
             self.save_or_sync_weights()
 
     def play(self, num_episodes, batch_size=1, min_replay_samples=1, print_progess_frequency=5, training=True, train_timing='on_episode_start', train_every_nstep=1, repeat_train=1,
-             need_render=True,**kwargs):
+             need_render=True, **kwargs):
         if train_timing not in ['on_episode_start', 'on_episode_end', 'on_step_end']:
             raise ValueError('Only on_episode_start,on_step_end are valid  train_timing options')
+        try:
+            if training:
+                self.actor.train()
+                self.critic.train()
+            else:
+                self.actor.eval()
+                self.critic.eval()
+            if self.use_experience_replay:
+                self.collect_samples(min_replay_samples=min_replay_samples)
+            elif train_timing == 'on_episode_start':
+                self.collect_samples(min_replay_samples=1, need_render=True if self.replay_unit == 'episode' else False)
+                print('start train....')
 
-        if training:
-            self.actor.train()
-            self.critic.train()
-        else:
-            self.actor.eval()
-            self.critic.eval()
-        if self.use_experience_replay:
-            self.collect_samples(min_replay_samples=min_replay_samples)
-        elif train_timing == 'on_episode_start':
-            self.collect_samples(min_replay_samples=1, need_render=True if self.replay_unit == 'episode' else False)
-            print('start train....')
+            self.train_every_nstep = train_every_nstep
+            self.total_epoch = num_episodes
+            self.total_reward = 0
+            self.t = 0
+            self.current_epoch = 0
+            self.train_steps = -1
+            self.current_screen = None
+            self.prev_screen = None
 
-        self.train_every_nstep = train_every_nstep
-        self.total_epoch = num_episodes
-        self.total_reward = 0
-        self.t = 0
-        self.current_epoch = 0
-        self.train_steps = -1
-        self.current_screen = None
-        self.prev_screen = None
+            if hasattr(self.env, 'recording_enabled'):
+                self.env.recording_enabled = True
+            for i_episode in range(num_episodes):
+                self.current_epoch = i_episode
+                self.current_batch = -1
+                self.frame_steps = -1
+                self.do_on_epoch_start()
 
-        if hasattr(self.env, 'recording_enabled'):
-            self.env.recording_enabled = True
-        for i_episode in range(num_episodes):
-            self.current_epoch = i_episode
-            self.current_batch = -1
-            self.frame_steps = -1
-            self.do_on_epoch_start()
+                if training and train_timing == 'on_episode_start' and i_episode % train_every_nstep == 0:
+                    self.training_model(i_episode, 0, num_episodes=num_episodes, repeat_train=repeat_train, train_timing=train_timing, batch_size=batch_size)
+                #self.state = self.env.reset()
 
-            if training and train_timing == 'on_episode_start' and i_episode % train_every_nstep == 0:
-                self.training_model(i_episode, 0, num_episodes=num_episodes, repeat_train=repeat_train, train_timing=train_timing, batch_size=batch_size)
-            self.state = self.env.reset()
+                self.total_rewards = 0
+                state = self.get_observation()
+                self.current_screen = state
+                self.prev_screen = state
+                for t in count():
+                    self.frame_steps += 1
+                    self.current_batch += 1
+                    self.t = t
 
-            self.total_rewards = 0
-            state = self.get_observation()
-            self.current_screen = state
-            self.prev_screen = state
-            for t in count():
-                self.frame_steps += 1
-                self.current_batch += 1
-                self.t = t
+                    # # Train on_step_start
+                    # if training and train_timing == 'on_step_start' and t % train_every_nstep == 0:
+                    #     self.training_model(i_episode, t,num_episodes=num_episodes, repeat_train=repeat_train, batch_size=batch_size)
 
-                # # Train on_step_start
-                # if training and train_timing == 'on_step_start' and t % train_every_nstep == 0:
-                #     self.training_model(i_episode, t,num_episodes=num_episodes, repeat_train=repeat_train, batch_size=batch_size)
+                    action = self.select_action(self.get_observation(), model_only=True)
+                    observation, reward, done, info = self.get_rewards(action)
 
+                    self.total_rewards += reward
 
-                action = self.select_action(self.get_observation(), model_only=True)
-                observation, reward, done, info = self.get_rewards(action)
+                    next_state = self.get_observation() if not done else zeros((1, 3, 84, 84))
+                    self.prev_screen = self.current_screen
+                    self.current_screen = next_state
 
-                self.total_rewards += reward
+                    if need_render:
+                        self.env.render()
 
-                next_state = self.get_observation() if not done else zeros((1, 3, 84, 84))
-                self.prev_screen = self.current_screen
-                self.current_screen = next_state
-
-                if need_render:
-                    self.env.render()
-
-                # Train on_step_end
-                if training and train_timing == 'on_step_end' and ((self.frame_steps + 1) % train_every_nstep == 0 or done) and self.frame_steps > 5:
-                    try:
-                        self.training_model(i_episode, t, num_episodes=num_episodes, done=done or complete, repeat_train=repeat_train, train_timing=train_timing)
-                    except Exception as e:
-                        print(e)
-
-                complete = self.episode_complete_criteria()
-
-                state = next_state
-
-                if done or complete:
                     # Train on_step_end
-                    if training and train_timing == 'on_episode_end' and ((self.frame_steps + 1) % train_every_nstep == 0 or done) and self.frame_steps > 5:
+                    if training and train_timing == 'on_step_end' and ((self.frame_steps + 1) % train_every_nstep == 0 or done) and self.frame_steps > 5:
                         try:
                             self.training_model(i_episode, t, num_episodes=num_episodes, done=done or complete, repeat_train=repeat_train, train_timing=train_timing)
                         except Exception as e:
                             print(e)
 
-                    self.epoch_metric_history.collect('rewards', i_episode, float(self.total_rewards))
-                    self.epoch_metric_history.collect('t', i_episode, float(t + 1))
-                    if self.use_experience_replay:
-                        self.epoch_metric_history.collect('replay_buffer_utility', i_episode, float(len(self.memory)) / self.memory.capacity)
+                    complete = self.episode_complete_criteria()
 
-                    self.do_on_epoch_end()
-                    if print_progess_frequency == 1 or ((i_episode + 1) % print_progess_frequency == 0):
-                        self.print_epoch_progress()
+                    state = next_state
 
-                    # 定期繪製損失函數以及評估函數對時間的趨勢圖
-                    if i_episode > 0 and (i_episode + 1) % (5 * print_progess_frequency) == 0:
-                        keywords = ['reward', 'critic', 'actor', 'loss', 'score']
-                        metrics_names = [k for k in self.epoch_metric_history.keys() if any([w in k.lower() for w in keywords]) or k.lower() == 't']
-                        loss_metric_curve(self.epoch_loss_history, self.epoch_metric_history, metrics_names=metrics_names, calculate_base='epoch',
-                                          imshow=True)
+                    if done or complete:
+                        # Train on_step_end
+                        if training and train_timing == 'on_episode_end' and ((self.frame_steps + 1) % train_every_nstep == 0 or done) and self.frame_steps > 5:
+                            try:
+                                self.training_model(i_episode, t, num_episodes=num_episodes, done=done or complete, repeat_train=repeat_train, train_timing=train_timing)
+                            except Exception as e:
+                                print(e)
 
-                    if self.task_complete_criteria():
-                        self.save_model(save_path=self.training_context['save_path'])
-                        print('episode {0} meet task complete criteria, training finish! '.format(i_episode))
-                        return True
+                        self.epoch_metric_history.collect('rewards', i_episode, float(self.total_rewards))
+                        self.epoch_metric_history.collect('t', i_episode, float(t + 1))
+                        if self.use_experience_replay:
+                            self.epoch_metric_history.collect('replay_buffer_utility', i_episode, float(len(self.memory)) / self.memory.capacity)
 
-                    break
+                        self.do_on_epoch_end()
+                        if print_progess_frequency == 1 or ((i_episode + 1) % print_progess_frequency == 0):
+                            self.print_epoch_progress()
 
-        print('Complete')
-        self.env.render()
-        self.env.close()
+                        # 定期繪製損失函數以及評估函數對時間的趨勢圖
+                        if i_episode > 0 and (i_episode + 1) % (5 * print_progess_frequency) == 0:
+                            keywords = ['reward', 'critic', 'actor', 'loss', 'score']
+                            metrics_names = [k for k in self.epoch_metric_history.keys() if any([w in k.lower() for w in keywords]) or k.lower() == 't']
+                            loss_metric_curve(self.epoch_loss_history, self.epoch_metric_history, metrics_names=metrics_names, calculate_base='epoch',
+                                              imshow=True)
+
+                        if self.task_complete_criteria():
+                            self.save_model(save_path=self.training_context['save_path'])
+                            print('episode {0} meet task complete criteria, training finish! '.format(i_episode))
+                            return True
+
+                        break
+            print('Complete')
+
+        except Exception as e:
+            print(e)
+            PrintException()
+            self.actor.save_model(save_path=self.actor.training_context['save_path'])
+            self.critic.save_model(save_path=self.critic.training_context['save_path'])
+            self.env.close()
 
     def learn(self, num_episodes, batch_size=1, min_replay_samples=1, print_progess_frequency=5, train_timing='on_episode_start', train_every_nstep=1, repeat_train=1,
-              need_render=True,**kwargs):
-        self.play(num_episodes=num_episodes, batch_size=batch_size, min_replay_samples=min_replay_samples, print_progess_frequency=print_progess_frequency, training=True,
-                  train_timing=train_timing, train_every_nstep=train_every_nstep,
-                  repeat_train=repeat_train, need_render=need_render)
+              need_render=True, **kwargs):
+        if ctx.enable_tensorboard:
+            for k, v in self.training_context.items():
+                if isinstance(v, HistoryBase):
+                    v.training_name = self.name
+
+            self.training_context['training_name'] = self.name
+            self.training_context['summary_writer'] = ctx.summary_writer
+
+            make_dir_if_need(os.path.join(working_directory, 'Logs'))
+
+
+            t1 = threading.Thread(target=launchTensorBoard, args=([]))
+            t1.setDaemon(True)
+            t1.start()
+            open_browser('http://{0}:{1}/'.format(ctx.tensorboard_server,ctx.tensorboard_port),5)
+        if ctx.enable_mlflow:
+            ctx.mlflow_logger.start_run(run_id=self.execution_id)
+            open_browser('http://{0}:{1}/'.format(ctx.mlflow_server, ctx.mlflow_port), 5)
+
+        self.training_context['repeat_train'] = repeat_train
+        self.training_context['train_timing'] = train_timing
+        self.training_context['train_every_nstep'] = train_every_nstep
+        self.training_context['min_replay_samples'] = min_replay_samples
+        self.training_context['batch_size'] = batch_size
+
+        self.do_on_training_start()
+
+        try:
+            self.play(num_episodes=num_episodes, batch_size=batch_size, min_replay_samples=min_replay_samples, print_progess_frequency=print_progess_frequency, training=True,
+                      train_timing=train_timing, train_every_nstep=train_every_nstep,
+                      repeat_train=repeat_train, need_render=need_render)
+        except Exception as e:
+            print(e)
+            PrintException()
 
     def resume(self, num_episodes=3000, **kwargs):
         pass
@@ -807,14 +900,57 @@ class ActorCriticPolicy(MuiltiNetwork):
     def data_preprocess(self, img_data):
         return img_data
 
+    def do_on_training_start(self):
+        for callback in self.callbacks:
+            callback.on_training_start(self.__dict__)
+        self.actor.train()
+        self.critic.train()
+
+    def do_on_training_end(self):
+        for callback in self.callbacks:
+            callback.on_training_end(self.__dict__)
+
+        self.save_model()
+        self.eval()
+
     def do_on_batch_end(self):
         self.training_context['time_batch_progress'] += (time.time() - self.training_context['time_batch_start'])
         self.training_context['time_epoch_progress'] += (time.time() - self.training_context['time_batch_start'])
-        if (self.training_context['steps'] + 1) % _session.epoch_equivalent == 0:
-            if self.warmup > 0 and self.warmup == (self.training_context['steps'] + 1) // _session.epoch_equivalent:
+        if (self.training_context['steps'] + 1) % ctx.epoch_equivalent == 0:
+            if self.warmup > 0 and self.warmup == (self.training_context['steps'] + 1) // ctx.epoch_equivalent:
                 self.adjust_learning_rate(self.training_context['base_lr'])
                 self.warmup = 0
 
+    def with_tensorboard(self):
+        make_dir_if_need(os.path.join(working_directory, 'Logs'))
+        # check weather have tensorboard
+        if get_backend() == 'pytorch':
+            try:
+                from trident.loggers.pytorch_tensorboard import SummaryWriter
+                ctx.try_enable_tensorboard(SummaryWriter(os.path.join(working_directory, 'Logs')))
+
+            except Exception as e:
+                print('Tensorboard initialize failed, please check the installation status about Tensorboard.')
+                print(e)
+                PrintException()
+        elif get_backend() == 'tensorflow':
+            try:
+                from trident.loggers.tensorflow_tensorboard import SummaryWriter
+                ctx.try_enable_tensorboard(SummaryWriter(os.path.join(working_directory, 'Logs')))
+
+            except Exception as e:
+                print('Tensorboard initialize failed, please check the installation status about Tensorboard.')
+                print(e)
+                PrintException()
+        return self
+
+
+    def with_mlflow(self):
+        from trident.loggers.mlflow_logger import MLFlowLogger
+        ctx.try_enable_mlflow(MLFlowLogger())
+
+        return self
+
 
 class DqnPolicy(PolicyBase):
     """The base class for any RL policy.
@@ -923,7 +1059,7 @@ class DqnPolicy(PolicyBase):
             self.save_model(save_path=self.training_context['save_path'])
 
     def learn(self, num_episodes, batch_size=1, min_replay_samples=1, print_progess_frequency=5, train_timing='on_episode_start', train_every_nstep=1, repeat_train=1,
-              accumulate_grads=False):
+              accumulate_grads=False,**kwargs):
         self.play(num_episodes=num_episodes, batch_size=batch_size, min_replay_samples=min_replay_samples, print_progess_frequency=print_progess_frequency, training=True,
                   train_timing=train_timing, train_every_nstep=train_every_nstep,
                   repeat_train=repeat_train, need_render=True)
@@ -1107,7 +1243,7 @@ class A2CPolicy(ActorCriticPolicy):
 
     def play(self, num_episodes, batch_size=1, min_replay_samples=1, print_progess_frequency=5, training=True, train_timing='on_episode_start', train_every_nstep=None,
              repeat_train=1,
-             need_render=True):
+             need_render=True,**kwargs):
         if train_timing not in ['on_episode_start', 'on_episode_end', 'on_step_end']:
             raise ValueError('Only on_episode_start,on_step_end are valid  train_timing options')
         if train_every_nstep is None:
@@ -1183,9 +1319,10 @@ class PPOPolicy(ActorCriticPolicy):
     """Proximal Policy Optimization
     """
 
-    def __init__(self, actor: Layer, critic: Layer, env: gym.Env,state_shape=None ,use_experience_replay=False, memory_length: int = 10000
+    def __init__(self, actor: Layer, critic: Layer, env: gym.Env, state_shape=None, use_experience_replay=False, memory_length: int = 10000
                  , gamma=0.9, beta=0.01, use_gae=False, gae_lambda=0.95, normalize_advantages=False, clip_grad_norm=0.5, name='ppo') -> None:
-        super(PPOPolicy, self).__init__(actor=actor, critic=critic, env=env,state_shape=state_shape, action_strategy=ActionStrategy.OnPolicy, gamma=gamma, use_experience_replay=use_experience_replay,
+        super(PPOPolicy, self).__init__(actor=actor, critic=critic, env=env, state_shape=state_shape, action_strategy=ActionStrategy.OnPolicy, gamma=gamma,
+                                        use_experience_replay=use_experience_replay,
                                         replay_unit='episode',
                                         memory_length=memory_length, name=name)
         self.state = self.env.reset()
@@ -1238,6 +1375,8 @@ class PPOPolicy(ActorCriticPolicy):
         return int(action.item())
 
     def get_rewards(self, action):
+        if hasattr(self,'done') and self.done:
+            self.env.reset()
         observation, reward, done, info = self.env.step(action)
 
         self.state = observation
@@ -1266,7 +1405,7 @@ class PPOPolicy(ActorCriticPolicy):
             Ratio2 = []
             LogProbs = []
 
-            #maske lstm stateful
+            # maske lstm stateful
             for module in self.actor.model.modules():
                 class_name = module.__class__.__name__.lower()
                 if 'lstm' in class_name or 'gru' in class_name:
@@ -1296,21 +1435,21 @@ class PPOPolicy(ActorCriticPolicy):
                 clipped_gaes = self.calculate_gae(clipped_value, V_, R, done_pool, self.gamma, self.gae_lambda)
                 advantages = max(gaes, clipped_gaes)
                 if self.normalize_advantages:
-                    #only divide std
+                    # only divide std
                     advantages = advantages / (advantages.std() + 1e-5)
             else:
                 advantages = max(V - R, clipped_value - R)
 
             critic_loss = (advantages ** 2).mean()
 
-            #need forward calculate for lstm-based actor
+            # need forward calculate for lstm-based actor
             for i in range(len(reward_pool)):
                 new_probs = self.actor(state_pool[i])
                 new_m = Categorical(probs=new_probs)
                 new_log_prob = new_m.log_prob(to_tensor(action_pool[i]))
                 LogProbs.insert(0, new_log_prob)
 
-                ratio1 = exp(new_log_prob - log_prob_pool[i])
+                ratio1 = exp(new_log_prob - log_prob_pool[i].detach())
                 ratio2 = clip(ratio1, 1.0 - 0.2, 1.0 + 0.2)
                 Ratio1.insert(0, ratio1)
                 Ratio2.insert(0, ratio2)
@@ -1322,17 +1461,21 @@ class PPOPolicy(ActorCriticPolicy):
             self.actor.batch_metric_history.collect('ratio1', self.train_steps, Ratio1.copy().detach().mean())
             self.actor.batch_metric_history.collect('ratio2', self.train_steps, Ratio2.copy().detach().mean())
 
-            surr1 = Ratio1 * (advantages.detach())
-            surr2 = Ratio2 * (advantages.detach())
-            actor_loss = None
-            actor_loss = (-1 * minimum(surr1,surr2)).mean()
+            if self.done == True:
+
+                actor_loss = -1 * (LogProbs * (advantages.detach())).mean()
+            else:
+                surr1 = Ratio1 * (advantages.detach())
+                surr2 = Ratio2 * (advantages.detach())
+                actor_loss = None
+                actor_loss = (-1 * minimum(surr1, surr2)).mean()
 
             return critic_loss, actor_loss, advantages, R, V
         except Exception as e:
             print(e)
 
     def save_or_sync_weights(self):
-        if self.current_epoch <=1 or  (self.train_steps + 1) % 5 == 0:
+        if self.current_epoch <= 1 or (self.train_steps) % self.accumulation_steps  == 0:
             self.save_model(save_path=self.training_context['save_path'])
             self.critic.model.to(get_device())
             self.actor.model.to(get_device())
@@ -1340,7 +1483,7 @@ class PPOPolicy(ActorCriticPolicy):
             self.old_actor.eval()
             self.actor.model.train()
 
-    def training_model(self, current_episode=0, current_step=0, num_episodes=100, train_timing='on_step_end', done=False, repeat_train=1):
+    def training_model(self, current_episode=0, current_step=0, num_episodes=100, train_timing='on_step_end', done=False, repeat_train=1, **kwargs):
 
         self.critic.batch_loss_history.regist('critic_loss')
         self.actor.batch_loss_history.regist('actor_loss')
@@ -1361,10 +1504,10 @@ class PPOPolicy(ActorCriticPolicy):
 
                 if self.done == True and len(self.memory) > 5 and self.current_epoch > 0:
                     for module in self.actor.model.modules():
-                        class_name=module.__class__.__name__.lower()
+                        class_name = module.__class__.__name__.lower()
                         if 'lstm' in class_name or 'gru' in class_name:
-                            if hasattr(module,'stateful'):
-                                module.stateful=False
+                            if hasattr(module, 'stateful'):
+                                module.stateful = False
 
                     for i in range(8):
                         try:
@@ -1372,8 +1515,6 @@ class PPOPolicy(ActorCriticPolicy):
                             self.current_batch += 1
                             self.steps = self.train_steps
 
-
-
                             # self.do_on_batch_start()
 
                             batch_data = self.memory.get_samples(builtins.min(64, len(self.memory)))
@@ -1386,7 +1527,7 @@ class PPOPolicy(ActorCriticPolicy):
                             batch_return = to_tensor(batch_return)
                             batch_advantages = (batch_return.detach() - batch_value)
 
-                            batch_critic_loss =(batch_advantages ** 2).mean()
+                            batch_critic_loss = (batch_advantages ** 2).mean()
                             self.critic.current_loss = batch_critic_loss
                             self.critic.batch_loss_history.collect('critic_loss', self.train_steps, batch_critic_loss.copy().detach())
                             self.critic.batch_metric_history.collect('critic_rmse', self.train_steps, (batch_critic_loss.copy().detach()).sqrt())
@@ -1394,8 +1535,8 @@ class PPOPolicy(ActorCriticPolicy):
                             self.critic.current_loss = 0.0
                             batch_actor_loss = to_tensor(0.0)
                             for j in range(len(batch_state)):
-                                # old_batch_probs = self.old_actor(to_tensor(batch_state[j]))
-                                # old_m = Categorical(probs=old_batch_probs)
+                                old_batch_probs = self.old_actor(to_tensor(batch_state[j]))
+                                old_m = Categorical(probs=old_batch_probs)
                                 old_batch_probs = batch_log_prob[j].long().detach()
                                 new_batch_probs = self.actor(to_tensor(batch_state[j]))
                                 new_m = Categorical(probs=new_batch_probs)
@@ -1406,7 +1547,7 @@ class PPOPolicy(ActorCriticPolicy):
                                 cliprange = 0.2
                                 surr1 = ratio * (batch_advantages[j].detach())
                                 surr2 = clip(ratio, 1.0 - cliprange, 1.0 + cliprange) * (batch_advantages[j].detach())
-                                batch_actor_loss = batch_actor_loss - (minimum(surr1,surr2)).mean()
+                                batch_actor_loss = batch_actor_loss - (minimum(surr1, surr2)).mean()
                             self.actor.current_loss = batch_actor_loss / len(batch_state)
                             self.actor.batch_loss_history.collect('actor_loss', self.train_steps, batch_actor_loss.copy() / len(batch_state))
                             # self.actor.batch_loss_history.collect('entropy_loss', self.train_steps, (-0.01 * entropy_loss).copy().detach() / (len(state_pool) - 1))
@@ -1453,10 +1594,6 @@ class PPOPolicy(ActorCriticPolicy):
                         # keep = list(filter(not_negative, list(sorted((set(keep))))))
                         # for idx in keep:
 
-
-
-
-
                         self.actor.current_loss = actor_loss
                         self.actor.batch_loss_history.collect('actor_loss', self.train_steps, actor_loss.copy().detach())
                         # self.actor.batch_loss_history.collect('entropy_loss', self.train_steps, (-0.01 * entropy_loss).copy().detach() / (len(state_pool) - 1))
@@ -1464,18 +1601,17 @@ class PPOPolicy(ActorCriticPolicy):
                         self.update_actor(self.current_epoch, self.train_steps, self.total_epoch, 1, 'on_step_end', self.done, accumulate_grads=False)
                         self.actor.current_loss = 0.0
                         # self.actor.print_batch_progress(1)
-                        if  self.done==True:
+                        if self.done == True:
                             self.save_or_sync_weights()
-                            #self.memory.reset()
+                            # self.memory.reset()
 
                         for idx in list(range(len(state_pool)))[::-1]:
-                            if self.rollout.reward[idx]!=0:
-                                self.memory.collect('return', advantages[idx].item()+V[idx].item())
+                            if self.rollout.reward[idx] != 0:
+                                self.memory.collect('return', advantages[idx].item() + V[idx].item())
                                 self.memory.collect('state', to_numpy(state_pool[idx]))
                                 self.memory.collect('action', action_pool[idx])
                                 self.memory.collect('log_prob', log_prob_pool[idx])
 
-
                         self.rollout.reset()
                         del log_prob_pool
                         del state_pool
